@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2015 David Gobbi
+  Copyright (c) 2012-2019 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -14,7 +14,7 @@
 #ifndef progress_h
 #define progress_h
 
-#include <vtkCommand.h>
+#include "vtkCommand.h"
 
 // Capture progress events
 class ProgressObserver : public vtkCommand
@@ -22,8 +22,12 @@ class ProgressObserver : public vtkCommand
 public:
   static ProgressObserver *New() { return new ProgressObserver; }
   vtkTypeMacro(ProgressObserver,vtkCommand);
-  virtual void Execute(
-    vtkObject *caller, unsigned long eventId, void *callData);
+#ifdef VTK_OVERRIDE
+  void Execute(
+    vtkObject *caller, unsigned long eventId, void *callData) VTK_OVERRIDE;
+#else
+  void Execute(vtkObject *caller, unsigned long eventId, void *callData);
+#endif
   void SetText(const char *text) { this->Text = text; }
 protected:
   ProgressObserver() : Stage(0), Anim(0), LastTime(0), Text("") {}
