@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2015 David Gobbi
+  Copyright (c) 2012-2019 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -14,6 +14,7 @@
 
 #include "readquery.h"
 
+#include "vtkDICOMFile.h"
 #include "vtkDICOMSequence.h"
 #include "vtkDICOMDictionary.h"
 
@@ -22,7 +23,6 @@
 #include <stdlib.h>
 
 #include <algorithm>
-#include <iostream>
 
 #ifdef _WIN32
 #include <windows.h>
@@ -30,50 +30,126 @@
 
 typedef vtkDICOMVR VR;
 
-// Prototype for function that reads one query key
-bool dicomcli_readkey_query(
-  const char *cp, vtkDICOMItem *query, QueryTagList *ql, bool qfile);
+namespace {
 
-// Build a tagpath
-vtkDICOMTagPath path_append(const vtkDICOMTagPath& tpath, vtkDICOMTag tag)
+// A class for reading a text file line-by-line.
+// It uses its own buffer, since the file is unbuffered.
+class LineReader
 {
-  vtkDICOMTagPath result(tag);
+public:
+  LineReader(vtkDICOMFile *file);
+  ~LineReader();
 
-  if (tpath.GetHead() != vtkDICOMTag())
+  size_t ReadLine(std::string *s);
+
+private:
+  vtkDICOMFile *File;
+  size_t BufferSize;
+  unsigned char *Buffer;
+  const unsigned char *Pointer;
+  const unsigned char *EndPointer;
+};
+
+LineReader::LineReader(vtkDICOMFile *file) :
+  File(file), BufferSize(4096)
+{
+  this->Buffer = new unsigned char [this->BufferSize];
+  this->Pointer = &this->Buffer[0];
+  this->EndPointer = this->Pointer;
+}
+
+LineReader::~LineReader()
+{
+  delete [] this->Buffer;
+  this->File->Close();
+}
+
+size_t LineReader::ReadLine(std::string *s)
+{
+  s->clear();
+
+  size_t total = 0;
+  while (!this->File->GetError() && !this->File->EndOfFile())
   {
-    if (tpath.HasTail())
+    if (this->Pointer == this->EndPointer)
     {
-      result = vtkDICOMTagPath(
-        tpath.GetHead(), tpath.GetIndex(),
-        tpath.GetTail().GetHead(), 0, tag);
+      this->Pointer = this->Buffer;
+      this->EndPointer = this->Buffer;
+      this->EndPointer += this->File->Read(this->Buffer, this->BufferSize);
+      if (this->Pointer == this->EndPointer)
+      {
+        break;
+      }
     }
-    else
+
+    const unsigned char *ucp = this->Pointer;
+    const char *cp = reinterpret_cast<const char *>(ucp);
+    size_t l = 0;
+    while (ucp != this->EndPointer && *ucp != '\r' && *ucp != '\n')
     {
-      result = vtkDICOMTagPath(
-        tpath.GetHead(), 0, tag);
+      ucp++;
+      l++;
+    }
+    this->Pointer = ucp;
+    if (ucp == this->EndPointer)
+    {
+      // append and continue
+      s->append(cp, l);
+      total += l;
+    }
+    else if (*ucp == '\n')
+    {
+      // newline means end of line
+      this->Pointer++;
+      l++;
+      s->append(cp, l);
+      total += l;
+      break;
+    }
+    else if (*ucp == '\r')
+    {
+      // carriage return is end of line, also eat following newline
+      this->Pointer++;
+      l++;
+      if (this->Pointer == this->EndPointer)
+      {
+        s->append(cp, l);
+        total += l;
+        l = 0;
+        this->Pointer = this->Buffer;
+        this->EndPointer = this->Buffer;
+        this->EndPointer += this->File->Read(this->Buffer, this->BufferSize);
+        cp = reinterpret_cast<const char *>(this->Pointer);
+      }
+      if (this->Pointer != this->EndPointer && *this->Pointer == '\n')
+      {
+        this->Pointer++;
+        l++;
+      }
+      if (l != 0)
+      {
+        s->append(cp, l);
+        total += l;
+      }
+      break;
     }
   }
 
-  return result;
+  return total;
 }
 
+}
+
+// Prototype for function that reads one query key
+bool dicomcli_readkey_query(
+  const char *cp, vtkDICOMItem *query, QueryTagList *ql,
+  bool ql_unique, bool qfile);
+
 // Read a query file
 bool dicomcli_readquery(
-  const char *fname, vtkDICOMItem *query, QueryTagList *ql)
+  const char *fname, vtkDICOMItem *query, QueryTagList *ql, bool ql_unique)
 {
-#ifdef _WIN32
-  int cn = MultiByteToWideChar(CP_UTF8, 0, fname, -1, NULL, 0);
-  wchar_t *wfname = new wchar_t[cn];
-  MultiByteToWideChar(CP_UTF8, 0, fname, -1, wfname, cn);
-  ifstream f(wfname);
-  delete [] wfname;
-#else
-  ifstream f(fname);
-#endif
-  if (!f.good())
-  {
-    return false;
-  }
+  vtkDICOMFile f(fname, vtkDICOMFile::In);
 
   // Each query line is either:
   // # a comment
@@ -84,17 +160,17 @@ bool dicomcli_readquery(
   // GGGG,EEEE\GGGG,EEEE         # a tag nested within a sequence
 
   int lineNumber = 0;
-  while (f.good())
+  std::string line;
+  LineReader lr(&f);
+  while (lr.ReadLine(&line))
   {
-    std::string line;
-    std::getline(f, line);
     const char *cp = line.c_str();
     size_t n = line.size();
     lineNumber++;
 
     // strip leading whitespace
     size_t s = 0;
-    while (s < n && isspace(cp[s]))
+    while (s < n && (cp[s] & 0x80) == 0 && isspace(cp[s]))
     {
       s++;
     }
@@ -105,36 +181,40 @@ bool dicomcli_readquery(
       continue;
     }
 
-    if (!dicomcli_readkey_query(cp, query, ql, true))
+    if (!dicomcli_readkey_query(cp, query, ql, ql_unique, true))
     {
       fprintf(stderr, "Error %s line %d:\n", fname, lineNumber);
       return false;
     }
   }
 
-  return true;
+  return (f.GetError() == 0);
 }
 
 // If qfile is true, then key is being read from a query file
 bool dicomcli_readkey_query(
-  const char *cp, vtkDICOMItem *query, QueryTagList *ql, bool qfile)
+  const char *cp, vtkDICOMItem *query, QueryTagList *ql,
+  bool ql_unique, bool qfile)
 {
   // read the tag path
   vtkDICOMTagPath tagPath;
+  std::string creator;
   size_t tagDepth = 0;
   size_t s = 0;
   size_t lineStart = s;
   size_t n = strlen(cp);
   bool tagError = false;
-  while (!tagError && tagDepth < 3)
+  const int maxerrlen = 80;
+
+  // set the default character set to utf-8
+  vtkDICOMCharacterSet cs(vtkDICOMCharacterSet::ISO_IR_192);
+
+  while (!tagError)
   {
     // check for private creator in square brackets
-    size_t creatorStart = s;
-    size_t creatorEnd = s;
     if (cp[s] == '[')
     {
-      s++;
-      creatorStart = s;
+      size_t t = ++s;
       while (s < n && cp[s] != ']')
       {
         s++;
@@ -145,26 +225,24 @@ bool dicomcli_readkey_query(
         tagError = true;
         continue;
       }
-      creatorEnd = s;
-      s++;
+      creator.assign(&cp[t], s++ - t);
     }
 
-    std::string creator(&cp[creatorStart], creatorEnd - creatorStart);
-
     // read the DICOM tag
     vtkDICOMTag tag(0x0000,0x0000);
     size_t tagStart = s;
     bool isHex = true;
     bool hasComma = false;
     size_t commaPos = 0;
-    while (s < n && (isalnum(cp[s]) || (cp[s] == ',' && !hasComma)))
+    while (s < n && (cp[s] & 0x80) == 0 &&
+           (isalnum(cp[s]) || (cp[s] == ',' && !hasComma)))
     {
       if (cp[s] == ',')
       {
         hasComma = true;
         commaPos = s - tagStart;
       }
-      else if (!isxdigit(cp[s]))
+      else if ((cp[s] & 0x80) != 0 || !isxdigit(cp[s]))
       {
         isHex = false;
       }
@@ -201,7 +279,7 @@ bool dicomcli_readkey_query(
     }
 
     // if creator, then resolve the tag now
-    if (creator.length() > 0)
+    if (creator.length() > 0 && (tag.GetGroup() & 0x0001) != 0)
     {
       if (tagDepth == 0)
       {
@@ -209,24 +287,40 @@ bool dicomcli_readkey_query(
       }
       else
       {
-        vtkDICOMSequence seq = query->GetAttributeValue(tagPath);
-        vtkDICOMItem item = seq.GetItem(0);
-        tag = item.ResolvePrivateTagForWriting(tag, creator);
-        vtkDICOMTag ctag(tag.GetGroup(), tag.GetElement() >> 8);
-        vtkDICOMTagPath ctagPath = path_append(tagPath, ctag);
-        query->SetAttributeValue(ctagPath, creator);
+        // Since "item" is copy-on-write, we want to avoid creating
+        // an unnecessary copy when the the creator element is added.
+        // So we get the item, add the creator element, then get
+        // the tag and value for that element, and let the item go
+        // out of scope so that its reference count is decremented
+        // (hopefully decremented to 1, so that the "copy" is not
+        // performed).  Then we add the creator element to the
+        // query data set (which may have become a copy due to
+        // copy-on-write, if the reference count was greater than one).
+        vtkDICOMTagPath ctagPath;
+        vtkDICOMValue cval;
+        {
+          // this must be in its own scope in order to work...
+          vtkDICOMSequence seq = query->Get(tagPath);
+          vtkDICOMItem item = seq.GetItem(0);
+          tag = item.ResolvePrivateTagForWriting(tag, creator);
+          vtkDICOMTag ctag(tag.GetGroup(), tag.GetElement() >> 8);
+          ctagPath = vtkDICOMTagPath(tagPath, 0, ctag);
+          cval = item.Get(ctag);
+        }
+        // add the creator element
+        query->Set(ctagPath, cval);
       }
     }
 
     // build the tag path
-    tagPath = path_append(tagPath, tag);
+    tagPath = vtkDICOMTagPath(tagPath, 0, tag);
 
     if (s < n && (cp[s] == '/' || cp[s] == '\\'))
     {
       // create an item for the next level of depth
-      if (!query->GetAttributeValue(tagPath).IsValid())
+      if (!query->Get(tagPath).IsValid())
       {
-        query->SetAttributeValue(tagPath, vtkDICOMSequence(1));
+        query->Set(tagPath, vtkDICOMSequence(1));
       }
       s++;
       tagDepth++;
@@ -238,7 +332,7 @@ bool dicomcli_readkey_query(
   }
 
   // if an error occurred while reading tag, skip to next line
-  if (tagError || tagDepth > 2)
+  if (tagError)
   {
     return false;
   }
@@ -261,7 +355,7 @@ bool dicomcli_readkey_query(
       if (!vr.IsValid() || vr == VR::OX || vr == VR::XS || vr == VR::UN)
       {
         int m = static_cast<int>(vrEnd - lineStart);
-        m = (m > 40 ? 40 : m);
+        m = (m > maxerrlen ? maxerrlen : m);
         fprintf(stderr, "Error: Unrecognized DICOM VR \"%*.*s\"\n",
            m, m, &cp[lineStart]);
         return false;
@@ -275,7 +369,7 @@ bool dicomcli_readkey_query(
   vtkDICOMTagPath tmpPath = tagPath;
   while (tmpPath.HasTail())
   {
-    pitem = pitem->GetAttributeValue(tag).GetSequenceData();
+    pitem = pitem->Get(tag).GetSequenceData();
     tmpPath = tmpPath.GetTail();
     tag = tmpPath.GetHead();
   }
@@ -303,7 +397,7 @@ bool dicomcli_readkey_query(
           ((dictvr == VR::XS && (vr == VR::SS || vr == VR::US)))))
     {
       int m = static_cast<int>(vrEnd - lineStart);
-      m = (m > 40 ? 40 : m);
+      m = (m > maxerrlen ? maxerrlen : m);
       fprintf(stderr, "Error: VR of \"%*.*s\" doesn't match dict VR of %s\n",
          m, m, &cp[lineStart], dictvr.GetText());
     }
@@ -312,7 +406,7 @@ bool dicomcli_readkey_query(
   if (!vr.IsValid() || vr == VR::UN)
   {
     int m = static_cast<int>(s - lineStart);
-    m = (m > 40 ? 40 : m);
+    m = (m > maxerrlen ? maxerrlen : m);
     fprintf(stderr, "Error: Unrecognized DICOM tag \"%*.*s\"\n",
             m, m, &cp[lineStart]);
     return false;
@@ -358,16 +452,16 @@ bool dicomcli_readkey_query(
     }
     else
     {
-      while (s < n && (!qfile || !isspace(cp[s])))
+      while (s < n && (!qfile || (cp[s] & 0x80) != 0 || !isspace(cp[s])))
       {
         s++;
       }
       valueEnd = s;
     }
   }
-  else if (s < n && (!qfile || !isspace(cp[s])))
+  else if (s < n && (!qfile || (cp[s] & 0x80) != 0 || !isspace(cp[s])))
   {
-    if (isgraph(cp[s]))
+    if ((cp[s] & 0x80) != 0 || isgraph(cp[s]))
     {
       fprintf(stderr, "Error: Illegal character \"%c\" after tag.\n", cp[s]);
     }
@@ -382,10 +476,10 @@ bool dicomcli_readkey_query(
   if (valueStart == valueEnd)
   {
     // only overwrite previous value if '=' was explicitly used
-    if (keyHasAssignment || !query->GetAttributeValue(tagPath).IsValid())
+    if (keyHasAssignment || !query->Get(tagPath).IsValid())
     {
       // empty value (always matches, always retrieved)
-      query->SetAttributeValue(tagPath, vtkDICOMValue(vr));
+      query->Set(tagPath, vtkDICOMValue(vr));
     }
   }
   else if (valueContainsQuotes)
@@ -400,18 +494,33 @@ bool dicomcli_readkey_query(
         t++;
       }
     }
-    query->SetAttributeValue(tagPath, vtkDICOMValue(vr, sval));
+    if (vr.HasSpecificCharacterSet())
+    {
+      query->Set(tagPath, vtkDICOMValue(vr, cs, sval));
+    }
+    else
+    {
+      query->Set(tagPath, vtkDICOMValue(vr, sval));
+    }
   }
   else
   {
-    query->SetAttributeValue(tagPath,
+    if (vr.HasSpecificCharacterSet())
+    {
+    query->Set(tagPath,
+      vtkDICOMValue(vr, cs, &cp[valueStart], valueEnd - valueStart));
+    }
+    else
+    {
+    query->Set(tagPath,
       vtkDICOMValue(vr, &cp[valueStart], valueEnd - valueStart));
+    }
   }
 
   // add the tag path to the list, if it isn't already there
   if (ql)
   {
-    if (std::find(ql->begin(), ql->end(), tagPath) == ql->end())
+    if (!ql_unique || std::find(ql->begin(), ql->end(), tagPath) == ql->end())
     {
       ql->push_back(tagPath);
     }
@@ -421,9 +530,9 @@ bool dicomcli_readkey_query(
 }
 
 bool dicomcli_readkey(
-  const char *cp, vtkDICOMItem *query, QueryTagList *ql)
+  const char *cp, vtkDICOMItem *query, QueryTagList *ql, bool ql_unique)
 {
-  return dicomcli_readkey_query(cp, query, ql, false);
+  return dicomcli_readkey_query(cp, query, ql, ql_unique, false);
 }
 
 bool dicomcli_looks_like_key(const char *cp)
@@ -464,7 +573,7 @@ bool dicomcli_looks_like_key(const char *cp)
       }
       digitrun = 0;
     }
-    else if (isxdigit(cp[l]))
+    else if ((cp[l] & 0x80) == 0 && isxdigit(cp[l]))
     {
       digitrun++;
     }
@@ -504,19 +613,7 @@ bool dicomcli_looks_like_key(const char *cp)
 bool dicomcli_readuids(
   const char *fname, vtkDICOMItem *query, QueryTagList *ql)
 {
-#ifdef _WIN32
-  int cn = MultiByteToWideChar(CP_UTF8, 0, fname, -1, NULL, 0);
-  wchar_t *wfname = new wchar_t[cn];
-  MultiByteToWideChar(CP_UTF8, 0, fname, -1, wfname, cn);
-  ifstream f(wfname);
-  delete [] wfname;
-#else
-  ifstream f(fname);
-#endif
-  if (!f.good())
-  {
-    return false;
-  }
+  vtkDICOMFile f(fname, vtkDICOMFile::In);
 
   // Basic file structure:
   // # one or more comments
@@ -526,23 +623,23 @@ bool dicomcli_readuids(
   QueryTagList ql2;
   std::string val;
   int lineNumber = 0;
-  while (f.good())
+  std::string line;
+  LineReader lr(&f);
+  while (lr.ReadLine(&line))
   {
-    std::string line;
-    std::getline(f, line);
     const char *cp = line.c_str();
     size_t n = line.size();
     lineNumber++;
 
     // strip leading whitespace
     size_t s = 0;
-    while (s < n && isspace(cp[s]))
+    while (s < n && (cp[s] & 0x80) == 0 && isspace(cp[s]))
     {
       s++;
     }
 
     // skip trailing whitespace
-    if (n > s && isspace(cp[n-1]))
+    if (n > s && (cp[n-1] & 0x80) == 0 && isspace(cp[n-1]))
     {
       --n;
     }
@@ -556,7 +653,7 @@ bool dicomcli_readuids(
     if (ql2.size() == 0)
     {
       // get the tag line, if not gotten yet
-      if (!dicomcli_readkey_query(cp, query, &ql2, true))
+      if (!dicomcli_readkey_query(cp, query, &ql2, true, true))
       {
         fprintf(stderr, "Error %s line %d: ", fname, lineNumber);
         fprintf(stderr, "Need Valid DICOM tag at top of file.\n");
@@ -584,7 +681,7 @@ bool dicomcli_readuids(
   {
     // add the key and value to the query
     vtkDICOMTagPath tagPath = ql2[0];
-    query->SetAttributeValue(tagPath, val);
+    query->Set(tagPath, val);
 
     if (ql && std::find(ql->begin(), ql->end(), tagPath) == ql->end())
     {
@@ -592,5 +689,26 @@ bool dicomcli_readuids(
     }
   }
 
-  return true;
+  return (f.GetError() == 0);
+}
+
+void dicomcli_error_helper(vtkDICOMMetaData *meta, int i)
+{
+  if (meta)
+  {
+    // print some useful identifying information about a DICOM file
+    fprintf(stderr, "Cannot read the DICOM file for the following entry:\n");
+    fprintf(stderr, "StudyInstanceUID=\"%s\",\n",
+      meta->Get(DC::StudyInstanceUID).AsString().c_str());
+    fprintf(stderr, "SeriesInstanceUID=\"%s\",\n",
+      meta->Get(DC::SeriesInstanceUID).AsString().c_str());
+    fprintf(stderr, "PatientID=\"%s\", StudyDate=\"%s\", StudyTime=\"%s\",\n",
+      meta->Get(DC::PatientID).AsString().c_str(),
+      meta->Get(DC::StudyDate).AsString().c_str(),
+      meta->Get(DC::StudyTime).AsString().c_str());
+    fprintf(stderr, "StudyID=\"%s\", SeriesNumber=\"%s\", InstanceNumber=\"%s\"\n",
+      meta->Get(DC::StudyID).AsString().c_str(),
+      meta->Get(DC::SeriesNumber).AsString().c_str(),
+      meta->Get(i, DC::InstanceNumber).AsString().c_str());
+  }
 }
