@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2016 David Gobbi
+  Copyright (c) 2012-2019 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -11,15 +11,19 @@
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
-/*! \class vtkDICOMApplyPalette
- *  \brief Apply palette to PALETTE COLOR images
+/**
+ * \class vtkDICOMApplyPalette
+ * \brief Apply palette to PALETTE COLOR images
  *
- *  If an image contains a palette LUT, then this filter will apply that
- *  LUT to the image and produce an RGB image.  Images without a palette
- *  LUT will pass through unchanged.  If the image uses a supplemental
- *  palette that only applies to a specific range of pixel values, then
- *  pixel values outside of that range will be mapped to greyscale via
- *  the WindowWidth and WindowCenter specified in the image.
+ * If an image contains a palette LUT, then this filter will apply that
+ * LUT to the image and produce an RGB image.  Images without a palette
+ * LUT will pass through unchanged.  If the image uses a supplemental
+ * palette that only applies to a specific range of pixel values, then
+ * pixel values outside of that range will be mapped to greyscale via
+ * the WindowWidth and WindowCenter specified in the image.
+ *
+ * \warning If this filter is used, then AutoRescaleOff() should be set on
+ * the reader to ensure that the reader does not rescale the data.
  */
 
 #ifndef vtkDICOMApplyPalette_h
@@ -39,24 +43,43 @@ public:
   vtkTypeMacro(vtkDICOMApplyPalette, vtkDICOMAlgorithm);
 
   //! Print information about this object.
-  virtual void PrintSelf(ostream& os, vtkIndent indent);
+#ifdef VTK_OVERRIDE
+  void PrintSelf(ostream& os, vtkIndent indent) VTK_OVERRIDE;
+#else
+  void PrintSelf(ostream& os, vtkIndent indent);
+#endif
 
 protected:
   vtkDICOMApplyPalette();
   ~vtkDICOMApplyPalette();
 
-  virtual int RequestInformation(
+#ifdef VTK_OVERRIDE
+  int RequestInformation(
+    vtkInformation* request, vtkInformationVector** inputVector,
+    vtkInformationVector* outputVector) VTK_OVERRIDE;
+
+  int RequestData(
+    vtkInformation* request, vtkInformationVector** inputVector,
+    vtkInformationVector* outputVector) VTK_OVERRIDE;
+
+  void ThreadedRequestData(
+    vtkInformation *request, vtkInformationVector **inputVector,
+    vtkInformationVector *outputVector, vtkImageData ***inData,
+    vtkImageData **outData, int ext[6], int id) VTK_OVERRIDE;
+#else
+  int RequestInformation(
     vtkInformation* request, vtkInformationVector** inputVector,
     vtkInformationVector* outputVector);
 
-  virtual int RequestData(
+  int RequestData(
     vtkInformation* request, vtkInformationVector** inputVector,
     vtkInformationVector* outputVector);
 
-  virtual void ThreadedRequestData(
+  void ThreadedRequestData(
     vtkInformation *request, vtkInformationVector **inputVector,
     vtkInformationVector *outputVector, vtkImageData ***inData,
     vtkImageData **outData, int ext[6], int id);
+#endif
 
   //! Container for the lookup tables used.
   vtkDICOMPerFilePalette *Palette;
@@ -65,8 +88,16 @@ protected:
   bool IsSupplemental;
 
 private:
+#ifdef VTK_DELETE_FUNCTION
   vtkDICOMApplyPalette(const vtkDICOMApplyPalette&) VTK_DELETE_FUNCTION;
   void operator=(const vtkDICOMApplyPalette&) VTK_DELETE_FUNCTION;
+#elif __cplusplus >= 201103L
+  vtkDICOMApplyPalette(const vtkDICOMApplyPalette&) = delete;
+  void operator=(const vtkDICOMApplyPalette&) = delete;
+#else
+  vtkDICOMApplyPalette(const vtkDICOMApplyPalette&);
+  void operator=(const vtkDICOMApplyPalette&);
+#endif
 };
 
 #endif // vtkDICOMApplyPalette_h
