@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2015 David Gobbi
+  Copyright (c) 2012-2019 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -33,7 +33,11 @@ public:
   vtkTypeMacro(vtkDICOMCTGenerator, vtkDICOMGenerator);
 
   //! Print information about this object.
-  virtual void PrintSelf(ostream& os, vtkIndent indent);
+#ifdef VTK_OVERRIDE
+  void PrintSelf(ostream& os, vtkIndent indent) VTK_OVERRIDE;
+#else
+  void PrintSelf(ostream& os, vtkIndent indent);
+#endif
 
   //! Generate an instance of one of the supported classes.
   /*!
@@ -41,7 +45,11 @@ public:
    *  information for a vtkImageData object, it will populate the
    *  attributes of the supplied vtkDICOMMetaData object.
    */
-  virtual bool GenerateInstance(vtkInformation *info);
+#ifdef VTK_OVERRIDE
+  bool GenerateInstance(vtkInformation *info) VTK_OVERRIDE;
+#else
+  bool GenerateInstance(vtkInformation *info);
+#endif
 
 protected:
   vtkDICOMCTGenerator();
@@ -57,8 +65,16 @@ protected:
   virtual bool GenerateCTInstance(vtkInformation *info);
 
 private:
+#ifdef VTK_DELETE_FUNCTION
   vtkDICOMCTGenerator(const vtkDICOMCTGenerator&) VTK_DELETE_FUNCTION;
   void operator=(const vtkDICOMCTGenerator&) VTK_DELETE_FUNCTION;
+#elif __cplusplus >= 201103L
+  vtkDICOMCTGenerator(const vtkDICOMCTGenerator&) = delete;
+  void operator=(const vtkDICOMCTGenerator&) = delete;
+#else
+  vtkDICOMCTGenerator(const vtkDICOMCTGenerator&);
+  void operator=(const vtkDICOMCTGenerator&);
+#endif
 };
 
 #endif // vtkDICOMCTGenerator_h
