@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2015 David Gobbi
+  Copyright (c) 2012-2019 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -14,7 +14,7 @@
 #ifndef vtkDICOMCompiler_h
 #define vtkDICOMCompiler_h
 
-#include <vtkObject.h>
+#include "vtkObject.h"
 #include "vtkDICOMModule.h" // For export macro
 
 
@@ -38,7 +38,11 @@ public:
   vtkTypeMacro(vtkDICOMCompiler, vtkObject);
 
   //! Print a summary of the contents of this object.
+#ifdef VTK_OVERRIDE
+  void PrintSelf(ostream& os, vtkIndent indent) VTK_OVERRIDE;
+#else
   void PrintSelf(ostream& os, vtkIndent indent);
+#endif
 
   //@{
   //! Set the file name.
@@ -269,8 +273,16 @@ protected:
   friend class vtkDICOMCompilerInternalFriendship;
 
 private:
+#ifdef VTK_DELETE_FUNCTION
   vtkDICOMCompiler(const vtkDICOMCompiler&) VTK_DELETE_FUNCTION;
   void operator=(const vtkDICOMCompiler&) VTK_DELETE_FUNCTION;
+#elif __cplusplus >= 201103L
+  vtkDICOMCompiler(const vtkDICOMCompiler&) = delete;
+  void operator=(const vtkDICOMCompiler&) = delete;
+#else
+  vtkDICOMCompiler(const vtkDICOMCompiler&);
+  void operator=(const vtkDICOMCompiler&);
+#endif
 };
 
 #endif /* vtkDICOMCompiler_h */
