@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2015 David Gobbi
+  Copyright (c) 2012-2019 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -116,7 +116,8 @@ vtkDICOMFile::vtkDICOMFile(const char *filename, Mode mode)
       if (wideFilename)
       {
         DWORD attr = GetFileAttributesW(wideFilename);
-        if ((attr & FILE_ATTRIBUTE_DIRECTORY) != 0)
+        if (attr != INVALID_FILE_ATTRIBUTES &&
+            (attr & FILE_ATTRIBUTE_DIRECTORY) != 0)
         {
           this->Error = FileIsDirectory;
         }
