@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2015 David Gobbi
+  Copyright (c) 2012-2019 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -14,7 +14,7 @@
 #ifndef vtkDICOMFileDirectory_h
 #define vtkDICOMFileDirectory_h
 
-#include <vtkSystemIncludes.h>
+#include "vtkSystemIncludes.h"
 #include "vtkDICOMModule.h" // For export macro
 
 #include <string> // Interface type
@@ -23,6 +23,13 @@
 class VTKDICOM_EXPORT vtkDICOMFileDirectory
 {
 public:
+  //! The access mode (input or output).
+  enum Mode
+  {
+    In,
+    Out
+  };
+
   //! Error codes.
   enum Code
   {
@@ -57,23 +64,46 @@ public:
 
   //@{
   //! Get the number of files and subdirectories in the directory.
-  int GetNumberOfFiles() { return this->NumberOfFiles; }
+  int GetNumberOfEntries() { return this->NumberOfEntries; }
 
   //! Get the name of the ith file or subdirectory.
-  const char *GetFile(int i);
+  /*!
+   *  The entries do not include "." or "..".
+   */
+  const char *GetEntry(int i);
 
   //! Check if the list entry is a directory.
   bool IsDirectory(int i);
 
+  //! Check if the list entry is special (a device, socket, or pipe).
+  bool IsSpecial(int i);
+
   //! Check if the list entry is a symbolic link.
   bool IsSymlink(int i);
 
+  //! Check if the list entry is a symbolic link that is broken.
+  /*!
+   *  A link is broken if it points to a location that does not exist,
+   *  or to a location that is in a directory that cannot be accessed.
+   *  IsSymlink() will always be true if IsBroken() is true.
+   */
+  bool IsBroken(int i);
+
   //! Check if the list entry has an attribute that marks it as hidden.
   bool IsHidden(int i);
   //@}
 
   //@{
-  //! Create a new directory with default permissions.
+  //! Test a directory for accessibility (static method).
+  /*!
+   *  The mode should be "In" or "Out" to indicate whether you intend to
+   *  read from or write to the directory.  A return value of zero means
+   *  the directory can be accessed, otherwise an error code will be
+   *  returned.
+   */
+  static int Access(const char *dirname, Mode mode);
+
+  //! Create a new directory with default permissions (static method).
   /*!
    *  This will create any intermediate directories, as well.  The return
    *  value is zero for success.  Otherwise, one of the error codes is
@@ -91,17 +121,22 @@ private:
   //! Add a directory entry.
   void AddEntry(const char *name, unsigned short flags, unsigned short mask);
 
-  //! Stat a file to set flags
+  //! Call stat() on a file to set flags
   void StatEntry(int i);
 
+  //! Call lstat() on a file to set flags
+  void LinkStatEntry(int i);
+
   struct Entry;
   static const unsigned int TypeDirectory = 1;
-  static const unsigned int TypeSymlink = 2;
-  static const unsigned int TypeHidden = 4;
+  static const unsigned int TypeSpecial = 2;
+  static const unsigned int TypeSymlink = 4;
+  static const unsigned int TypeBroken = 8;
+  static const unsigned int TypeHidden = 16;
 
   std::string Name;
   int Error;
-  int NumberOfFiles;
+  int NumberOfEntries;
   Entry *Entries;
 };
 
