@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2015 David Gobbi
+  Copyright (c) 2012-2019 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -14,7 +14,7 @@
 #ifndef vtkDICOMFileSorter_h
 #define vtkDICOMFileSorter_h
 
-#include <vtkObject.h>
+#include "vtkObject.h"
 #include "vtkDICOMModule.h" // For export macro
 
 class vtkStringArray;
@@ -33,7 +33,11 @@ class VTKDICOM_EXPORT vtkDICOMFileSorter : public vtkObject
 {
 public:
   vtkTypeMacro(vtkDICOMFileSorter,vtkObject);
+#ifdef VTK_OVERRIDE
+  void PrintSelf(ostream& os, vtkIndent indent) VTK_OVERRIDE;
+#else
   void PrintSelf(ostream& os, vtkIndent indent);
+#endif
   static vtkDICOMFileSorter *New();
 
   //@{
@@ -51,7 +55,7 @@ public:
   //! Set a list of file names to group and sort.
   /*!
    *  This method is mutually exclusive with SetInputFileName().
-   *  If you call ths method, then InputFileName will be set to NULL
+   *  If you call this method, then InputFileName will be set to NULL
    *  and vice versa.
    */
   void SetInputFileNames(vtkStringArray *input);
@@ -147,8 +151,16 @@ protected:
   void SetErrorCode(unsigned long e) { this->ErrorCode = e; }
 
 private:
+#ifdef VTK_DELETE_FUNCTION
   vtkDICOMFileSorter(const vtkDICOMFileSorter&) VTK_DELETE_FUNCTION;
   void operator=(const vtkDICOMFileSorter&) VTK_DELETE_FUNCTION;
+#elif __cplusplus >= 201103L
+  vtkDICOMFileSorter(const vtkDICOMFileSorter&) = delete;
+  void operator=(const vtkDICOMFileSorter&) = delete;
+#else
+  vtkDICOMFileSorter(const vtkDICOMFileSorter&);
+  void operator=(const vtkDICOMFileSorter&);
+#endif
 
   class StringArrayVector;
   struct FileInfo;
