@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2015 David Gobbi
+  Copyright (c) 2012-2019 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -14,8 +14,8 @@
 #ifndef vtkDICOMGenerator_h
 #define vtkDICOMGenerator_h
 
-#include <vtkObject.h>
-#include <vtkStdString.h> // For std::string
+#include "vtkObject.h"
+#include "vtkStdString.h" // For std::string
 #include "vtkDICOMModule.h" // For export macro
 #include "vtkDICOMTag.h" // For method parameter
 
@@ -48,7 +48,11 @@ public:
   vtkTypeMacro(vtkDICOMGenerator, vtkObject);
 
   //! Print information about this object.
-  virtual void PrintSelf(ostream& os, vtkIndent indent);
+#ifdef VTK_OVERRIDE
+  void PrintSelf(ostream& os, vtkIndent indent) VTK_OVERRIDE;
+#else
+  void PrintSelf(ostream& os, vtkIndent indent);
+#endif
 
   //@{
   //! Generate an instance of one of the supported classes.
@@ -121,6 +125,17 @@ public:
   vtkGetMacro(RescaleSlope, double);
   //@}
 
+  //@{
+  //! Set the number of overlays to generate.
+  /*!
+   *  This will cause overlay attributes to be copied from the source
+   *  meta data to the output meta data.  It is up to the writer to
+   *  ensure that the values are consistent with each other.
+   */
+  vtkSetMacro(NumberOfOverlays, int);
+  vtkGetMacro(NumberOfOverlays, int);
+  //@}
+
   //@{
   //! Set the matrix that places the image in DICOM patient coords.
   /*!
@@ -406,6 +421,9 @@ protected:
   double RescaleIntercept;
   double RescaleSlope;
 
+  //! The number of overlays to generate.
+  int NumberOfOverlays;
+
   //! The VTK scalar type of the data, set by InitializeMetaData().
   int ScalarType;
 
@@ -466,8 +484,16 @@ protected:
   vtkIntArray *RangeArray;
 
 private:
+#ifdef VTK_DELETE_FUNCTION
   vtkDICOMGenerator(const vtkDICOMGenerator&) VTK_DELETE_FUNCTION;
   void operator=(const vtkDICOMGenerator&) VTK_DELETE_FUNCTION;
+#elif __cplusplus >= 201103L
+  vtkDICOMGenerator(const vtkDICOMGenerator&) = delete;
+  void operator=(const vtkDICOMGenerator&) = delete;
+#else
+  vtkDICOMGenerator(const vtkDICOMGenerator&);
+  void operator=(const vtkDICOMGenerator&);
+#endif
 };
 
 #endif // vtkDICOMGenerator_h
