@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2015 David Gobbi
+  Copyright (c) 2012-2019 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -14,7 +14,7 @@
 #include "vtkDICOMLookupTable.h"
 #include "vtkDICOMMetaData.h"
 
-#include <vtkObjectFactory.h>
+#include "vtkObjectFactory.h"
 
 #include <string.h>
 
@@ -277,6 +277,84 @@ void vtkDICOMLookupTable::BuildStandardPalette(const char *cs)
     }
     this->Modified();
   }
+  else if (strcmp(cs, "SPRING") == 0)
+  {
+    int n = this->GetNumberOfColors();
+    unsigned char *cptr = this->WritePointer(0, n);
+    double rgba[4] = { 0.0, 0.0, 0.0, 1.0 };
+    for (int i = 0; i < n; i++)
+    {
+      double x = static_cast<double>(i)/(n - 1);
+      rgba[0] = 1.0;
+      rgba[1] = x;
+      rgba[2] = 1.0 - x;
+
+      cptr[0] = static_cast<unsigned char>(rgba[0]*255.0 + 0.5);
+      cptr[1] = static_cast<unsigned char>(rgba[1]*255.0 + 0.5);
+      cptr[2] = static_cast<unsigned char>(rgba[2]*255.0 + 0.5);
+      cptr[3] = static_cast<unsigned char>(rgba[3]*255.0 + 0.5);
+      cptr += 4;
+    }
+    this->Modified();
+  }
+  else if (strcmp(cs, "SUMMER") == 0)
+  {
+    int n = this->GetNumberOfColors();
+    unsigned char *cptr = this->WritePointer(0, n);
+    double rgba[4] = { 0.0, 0.0, 0.0, 1.0 };
+    for (int i = 0; i < n; i++)
+    {
+      double x = static_cast<double>(i)/(n - 1);
+      rgba[0] = x;
+      rgba[1] = 0.5 + 0.5*x;
+      rgba[2] = 0.4;
+
+      cptr[0] = static_cast<unsigned char>(rgba[0]*255.0 + 0.5);
+      cptr[1] = static_cast<unsigned char>(rgba[1]*255.0 + 0.5);
+      cptr[2] = static_cast<unsigned char>(rgba[2]*255.0 + 0.5);
+      cptr[3] = static_cast<unsigned char>(rgba[3]*255.0 + 0.5);
+      cptr += 4;
+    }
+  }
+  else if (strcmp(cs, "FALL") == 0)
+  {
+    int n = this->GetNumberOfColors();
+    unsigned char *cptr = this->WritePointer(0, n);
+    double rgba[4] = { 0.0, 0.0, 0.0, 1.0 };
+    for (int i = 0; i < n; i++)
+    {
+      double x = static_cast<double>(i)/(n - 1);
+      rgba[0] = 1.0;
+      rgba[1] = x;
+      rgba[2] = 0.0;
+
+      cptr[0] = static_cast<unsigned char>(rgba[0]*255.0 + 0.5);
+      cptr[1] = static_cast<unsigned char>(rgba[1]*255.0 + 0.5);
+      cptr[2] = static_cast<unsigned char>(rgba[2]*255.0 + 0.5);
+      cptr[3] = static_cast<unsigned char>(rgba[3]*255.0 + 0.5);
+      cptr += 4;
+    }
+    this->Modified();
+  }
+  else if (strcmp(cs, "WINTER") == 0)
+  {
+    int n = this->GetNumberOfColors();
+    unsigned char *cptr = this->WritePointer(0, n);
+    double rgba[4] = { 0.0, 0.0, 0.0, 1.0 };
+    for (int i = 0; i < n; i++)
+    {
+      double x = static_cast<double>(i)/(n - 1);
+      rgba[0] = 0.0;
+      rgba[1] = x;
+      rgba[2] = 1.0 - x*0.5;
+
+      cptr[0] = static_cast<unsigned char>(rgba[0]*255.0 + 0.5);
+      cptr[1] = static_cast<unsigned char>(rgba[1]*255.0 + 0.5);
+      cptr[2] = static_cast<unsigned char>(rgba[2]*255.0 + 0.5);
+      cptr[3] = static_cast<unsigned char>(rgba[3]*255.0 + 0.5);
+      cptr += 4;
+    }
+  }
 }
 
 //----------------------------------------------------------------------------
@@ -295,9 +373,14 @@ void vtkDICOMLookupTable::BuildImagePalette(
     DC::BluePaletteColorLookupTableData,
     DC::AlphaPaletteColorLookupTableData
   };
+  const DC::EnumType segTag[] = {
+    DC::SegmentedRedPaletteColorLookupTableData,
+    DC::SegmentedGreenPaletteColorLookupTableData,
+    DC::SegmentedBluePaletteColorLookupTableData,
+    DC::SegmentedAlphaPaletteColorLookupTableData
+  };
 
-  int isSigned = meta->GetAttributeValue(
-    fileIndex, DC::PixelRepresentation).AsInt();
+  int isSigned = meta->Get(fileIndex, DC::PixelRepresentation).AsInt();
   int minValue = VTK_INT_MAX;
   int maxValue = VTK_INT_MIN;
   int firstValue[4] = { 0, 0, 0, 0 };
@@ -305,10 +388,12 @@ void vtkDICOMLookupTable::BuildImagePalette(
   double divisor[4] = { 255.0, 255.0, 255.0, 255.0 };
   const unsigned short *spp[4] = { 0, 0, 0, 0 };
   const unsigned char *cpp[4] = { 0, 0, 0, 0 };
+  const unsigned short *segpp[4] = { 0, 0, 0, 0 };
+  unsigned int segn[4] = { 0, 0, 0, 0 };
+
   for (int j = 0; j < 4; j++)
   {
-    const vtkDICOMValue& v = meta->GetAttributeValue(
-      fileIndex, descTag[j]);
+    const vtkDICOMValue& v = meta->Get(fileIndex, descTag[j]);
     // Descriptor must have three values
     if (v.GetNumberOfValues() == 3)
     {
@@ -330,20 +415,29 @@ void vtkDICOMLookupTable::BuildImagePalette(
       divisor[j] = (nbits == 0 ? 1.0 : (1u << nbits) - 1.0);
 
       // Get the lookup table data and confirm its size
-      const vtkDICOMValue& d = meta->GetAttributeValue(
-        fileIndex, dataTag[j]);
-      if (d.GetVL() >= static_cast<unsigned int>(2*n))
+      const vtkDICOMValue& d = meta->Get(fileIndex, dataTag[j]);
+      if (d.IsValid())
       {
-        spp[j] = d.GetUnsignedShortData();
-      }
-      else if (d.GetVL() >= static_cast<unsigned int>(n))
-      {
-        cpp[j] = d.GetUnsignedCharData();
-        if (cpp[j] == 0)
+        if (d.GetVL() >= static_cast<unsigned int>(2*n))
         {
-          cpp[j] = reinterpret_cast<const unsigned char *>(
-            d.GetUnsignedShortData());
+          spp[j] = d.GetUnsignedShortData();
         }
+        else if (d.GetVL() >= static_cast<unsigned int>(n))
+        {
+          cpp[j] = d.GetUnsignedCharData();
+          if (cpp[j] == 0)
+          {
+            cpp[j] = reinterpret_cast<const unsigned char *>(
+              d.GetUnsignedShortData());
+          }
+        }
+      }
+      else
+      {
+        // Search for segmented table data
+        const vtkDICOMValue& s = meta->Get(fileIndex, segTag[j]);
+        segpp[j] = s.GetUnsignedShortData();
+        segn[j] = s.GetVL()/2;
       }
     }
   }
@@ -360,8 +454,8 @@ void vtkDICOMLookupTable::BuildImagePalette(
       for (int j = 0; j < 4; j++)
       {
         int k = i - firstValue[j];
-        k = (i >= firstValue[j] ? k : 0);
-        k = (i <= lastValue[j] ? k : lastValue[j] - firstValue[j]);
+        if (i < firstValue[j]) { k = 0; }
+        if (i > lastValue[j]) { k = lastValue[j] - firstValue[j]; }
         if (spp[j])
         {
           rgba[j] = spp[j][k] / divisor[j];
@@ -377,6 +471,149 @@ void vtkDICOMLookupTable::BuildImagePalette(
       cptr[3] = static_cast<unsigned char>(rgba[3]*255.0 + 0.5);
       cptr += 4;
     }
+
+    // Handle segmented lookup tables
+    cptr = this->WritePointer(0, maxValue - minValue + 1);
+    for (int j = 0; j < 4; j++)
+    {
+      if (segpp[j])
+      {
+        if (!vtkDICOMLookupTable::BuildSegmentedLUT(
+          segpp[j], segn[j], divisor[j], cptr+j, 4, maxValue-minValue+1))
+        {
+        static const char *names[] = { "Red", "Green", "Blue", "Alpha" };
+        vtkErrorMacro("Bad data in Segmented " << names[j]
+                      << " PaletteColorLookupTableData");
+        }
+      }
+    }
   }
+
   this->Modified();
 }
+
+//----------------------------------------------------------------------------
+bool vtkDICOMLookupTable::BuildSegmentedLUT(
+  const unsigned short *input, unsigned int n, double divisor,
+  unsigned char *lut, unsigned int stride, unsigned int count)
+{
+  // i is position in input, j is position in output
+  unsigned int i = 0;
+  unsigned int j = 0;
+
+  while (i+1 < n)
+  {
+    unsigned char opcode = input[i++];
+    if (opcode == 0)
+    {
+      // discrete segment
+      unsigned short l = input[i++];
+      for (unsigned short k = 0; k < l && i < n && j < count; k++, j++, i++)
+      {
+        double y = input[i]/divisor;
+        lut[j*stride] = static_cast<unsigned char>(y*255.0 + 0.5);
+      }
+    }
+    else if (opcode == 1)
+    {
+      // linear segment
+      unsigned short l = input[i++];
+      double y0 = 0.0;
+      if (j > 0)
+      {
+        // start of the ramp
+        y0 = lut[(j - 1)*stride]/255.0;
+      }
+      if (i < n)
+      {
+        // end of the ramp
+        double y1 = input[i++]/divisor;
+        // generate the ramp
+        for (unsigned short k = 0; k < l && j < count; k++, j++)
+        {
+          double y = y0 + (k + 1)*(y1 - y0)/l;
+          lut[j*stride] = static_cast<unsigned char>(y*255.0 + 0.5);
+        }
+      }
+    }
+    else if (opcode == 2)
+    {
+      // indirect segment
+      bool failed = true;
+      unsigned short nseg = input[i++];
+      if (i + 1 < n)
+      {
+        failed = false;
+        // offset to first segment to use
+        unsigned int offset = input[i++];
+        offset += input[i++] << 16;
+        // m is the number of input elements referred to, and
+        // l is the number of lut entries that will be generated
+        unsigned int m = 0;
+        unsigned int l = 0;
+        // compute m by looking at the segments
+        for (unsigned int k = 0; k < nseg && offset + m + 1 < n; k++)
+        {
+          unsigned short opcode2 = input[offset + m++];
+          if (opcode2 == 0)
+          {
+            // discrete
+            l += input[offset + m];
+            m += 1 + input[offset + m];
+          }
+          else if (opcode2 == 1)
+          {
+            // linear
+            l += input[offset + m];
+            m += 2;
+          }
+          else
+          {
+            // illegal opcode
+            failed = true;
+            break;
+          }
+        }
+
+        if (offset + m <= n)
+        {
+          // add the referenced segments via recursion
+          BuildSegmentedLUT(
+            &input[offset], m, divisor, &lut[j*stride], stride, count-j);
+
+          // increment the position in the output lut
+          if (j + l > count)
+          {
+            l = count - j;
+          }
+          j += l;
+        }
+      }
+      if (failed)
+      {
+        break;
+      }
+    }
+    else
+    {
+      // unrecognized opcode
+      break;
+    }
+  }
+
+  if (j < count)
+  {
+    // If write was incomplete, fill the remainder of the lookup table
+    // with alternating high and low to visually indicate that a
+    // problem occurred.
+    while (j < count)
+    {
+      lut[j*stride] = (j & 1)*255;
+      j++;
+    }
+
+    return false;
+  }
+
+  return true;
+}
