@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2016 David Gobbi
+  Copyright (c) 2012-2019 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -11,18 +11,20 @@
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
-/*! \class vtkDICOMReader
- *  \brief Read DICOM image files.
+/**
+ * \class vtkDICOMReader
+ * \brief Read DICOM image files.
  *
- *  This class reads a series of DICOM files into a vtkImageData object,
- *  and also provides access to the DICOM meta data for each file.
+ * This class reads a series of DICOM files into a vtkImageData object,
+ * and also provides access to the DICOM meta data for each file.
  */
 
 #ifndef vtkDICOMReader_h
 #define vtkDICOMReader_h
 
-#include <vtkImageReader2.h>
+#include "vtkImageReader2.h"
 #include "vtkDICOMModule.h" // For export macro
+#include "vtkDICOMCharacterSet.h" // For character sets
 
 class vtkIntArray;
 class vtkTypeInt64Array;
@@ -43,20 +45,39 @@ public:
   static vtkDICOMReader *New();
 
   //! Print information about this object.
-  virtual void PrintSelf(ostream& os, vtkIndent indent);
+#ifdef VTK_OVERRIDE
+  void PrintSelf(ostream& os, vtkIndent indent) VTK_OVERRIDE;
+#else
+  void PrintSelf(ostream& os, vtkIndent indent);
+#endif
 
+#ifdef VTK_OVERRIDE
   //@{
   //! Valid extensions for this file type.
-  virtual const char* GetFileExtensions() {
+  const char* GetFileExtensions() VTK_OVERRIDE {
     return ".dcm .dc"; }
 
   //! Return a descriptive name that might be useful in a GUI.
-  virtual const char* GetDescriptiveName() {
+  const char* GetDescriptiveName() VTK_OVERRIDE {
+    return "DICOM"; }
+
+  //! Return true if this reader can read the given file.
+  int CanReadFile(const char* filename) VTK_OVERRIDE;
+  //@}
+#else
+  //@{
+  //! Valid extensions for this file type.
+  const char* GetFileExtensions() {
+    return ".dcm .dc"; }
+
+  //! Return a descriptive name that might be useful in a GUI.
+  const char* GetDescriptiveName() {
     return "DICOM"; }
 
   //! Return true if this reader can read the given file.
   int CanReadFile(const char* filename);
   //@}
+#endif
 
   //@{
   //! Set the Stack ID of the stack to load, for named stacks.
@@ -104,15 +125,41 @@ public:
   //@{
   //! Get the meta data for the DICOM files.
   /*!
-   *  The GetAttributeValue() method of vtkDICOMMataData takes optional
-   *  file and frame indices, which specify the file and the frame within
-   *  that file to get the attribute from.  If you have a slice index rather
-   *  than a file index and frame index, then use the FileIndexArray and
+   *  The Get() method of vtkDICOMMataData takes optional file and frame
+   *  indices, which specify the file and the frame within that file to
+   *  get the attribute from.  If you have a slice index rather than a
+   *  file index and frame index, then use the FileIndexArray and
    *  FrameIndexArray to convert the slice index into file and frame indices.
    */
   vtkDICOMMetaData *GetMetaData() { return this->MetaData; }
   //@}
 
+  //@{
+  //! Set the character set to use if SpecificCharacterSet is missing.
+  /*!
+   *  Some DICOM files do not list a SpecificCharacterSet attribute, but
+   *  nevertheless use a non-ASCII character encoding.  This method can be
+   *  used to specify the character set in absence of SpecificCharacterSet.
+   *  This method will not take effect unless is is called before the image
+   *  is read.  If SpecificCharacterSet is present, the default will not
+   *  override it unless OverrideCharacterSet is true.
+   */
+  void SetDefaultCharacterSet(vtkDICOMCharacterSet cs);
+  vtkDICOMCharacterSet GetDefaultCharacterSet() {
+    return this->DefaultCharacterSet; }
+
+  //! Override the value stored in SpecificCharacterSet.
+  /*!
+   *  This method can be used if the SpecificCharacterSet attribute of a
+   *  file is incorrect.  It overrides the SpecificCharacterSet with the
+   *  DefaultCharacterSet.
+   */
+  vtkSetMacro(OverrideCharacterSet, bool);
+  vtkBooleanMacro(OverrideCharacterSet, bool);
+  bool GetOverrideCharacterSet() {
+    return this->OverrideCharacterSet; }
+  //@}
+
   //@{
   //! If the files have been pre-sorted, the sorting can be disabled.
   vtkGetMacro(Sorting, int);
@@ -168,15 +215,16 @@ public:
   //@}
 
   //@{
-  //! Turn off automatic rescaling of intensity values.
+  //! Turn off automatic rescaling of stored pixel values.
   /*!
-   *  By default, if the RescaleSlope and RescaleIntercept values differ
-   *  between slices (as occurs for all PET images and some CT images),
-   *  then the reader will adjust the pixel values for the slices so
-   *  that the same RescaleSlope and RescaleIntercept can be used for
-   *  all slices.  This adjustment is a lossy process, so a preferable
-   *  option is to call AutoRescaleOff() and use vtkDICOMApplyRescale
-   *  to apply the pixel value rescaling instead.
+   *  By default, if the RescaleSlope and RescaleIntercept attributes
+   *  are present in the DICOM data set, they will be used to rescale
+   *  stored pixel values according to the DICOM modality LUT operation.
+   *  The resulting values will be stored as 16-bit integers if possible,
+   *  but single-precision floats will be used if the potential output
+   *  range is greater than can be stored in 16 bits, or if either the
+   *  RescaleSlope or the RescaleIntercept is not an integer.
+   *  If AutoRescale is off, then the data is not rescaled.
    */
   vtkGetMacro(AutoRescale, int);
   vtkSetMacro(AutoRescale, int);
@@ -186,8 +234,14 @@ public:
   //@{
   //! Get the slope and intercept for rescaling the scalar values.
   /*!
-   *  These values allow calibration of the data to real values.
-   *  Use the equation v = u*RescaleSlope + RescaleIntercept.
+   *  If AutoRescale in On (which is the default), then GetRescaleSlope()
+   *  will return 1 and GetRescaleIntercept() will return zero.  Otherwise,
+   *  they will return the RescaleSlope and RescaleIntercept for the data
+   *  set. Use the equation v = u*RescaleSlope + RescaleIntercept.
+   *
+   *  Note that the returned values might not be valid if these attributes
+   *  are not the same for all slices in the series.  Use the meta data to
+   *  retrieve the per-slice slope and intercept.
    */
   double GetRescaleSlope() { return this->RescaleSlope; }
   double GetRescaleIntercept() { return this->RescaleIntercept; }
@@ -203,6 +257,26 @@ public:
   vtkMatrix4x4 *GetPatientMatrix() { return this->PatientMatrix; }
   //@}
 
+  //@{
+  //! Get the overlay.
+  /*!
+   *  The extent of the overlay will be the same as the main image.
+   *  For multiple overlays, each overlay will be stored in a different
+   *  bit. An 8-bit image will be used if there are eight or fewer
+   *  overlays, and an unsigned 16-bit image will be used if there are
+   *  more than eight overlays.
+   */
+  vtkImageData *GetOverlayOutput();
+  vtkAlgorithmOutput *GetOverlayOutputPort();
+  void SetOverlayOutput(vtkImageData *data);
+
+  //! Returns true if any overlays are present.
+  bool HasOverlay() { return (this->OverlayBitfield != 0); }
+
+  //! Returns a bitfield that indicates which overlays are present.
+  unsigned short GetOverlayBitfield() { return this->OverlayBitfield; }
+  //@}
+
   //@{
   //! Get a MedicalImageProperties object for this file.
   vtkMedicalImageProperties *GetMedicalImageProperties();
@@ -229,11 +303,60 @@ public:
   const char *GetMemoryRowOrderAsString();
   //@}
 
+  //@{
+  //! Set the scalar type of the output.
+  /*!
+   *  The default value is -1, which means that the input scalar type will
+   *  be used to set the output scalar type (accounting for any type
+   *  changes required by AutoRescale).  Allowed output scalar types are
+   *  VTK_SIGNED_CHAR, VTK_UNSIGNED_CHAR, VTK_SHORT, VTK_UNSIGNED_SHORT,
+   *  VTK_INT, VTK_UNSIGNED_INT, VTK_FLOAT, or VTK_DOUBLE.  The output
+   *  data values will be clamped to the limits of the output type.
+   */
+  vtkSetMacro(OutputScalarType, int);
+  vtkGetMacro(OutputScalarType, int);
+  //@}
+
+#ifndef __WRAP__
+  //@{
+  using Superclass::Update;
+  //! Update both the image and, if present, the overlay
+#ifdef VTK_OVERRIDE
+  void Update() VTK_OVERRIDE;
+#else
+  virtual void Update();
+#endif
+  //@}
+#endif
+
 protected:
   vtkDICOMReader();
   ~vtkDICOMReader();
 
+#ifdef VTK_OVERRIDE
   //@{
+  //! Entry point for all pipeline requests.
+  int ProcessRequest(
+    vtkInformation* request, vtkInformationVector** inputVector,
+    vtkInformationVector* outputVector) VTK_OVERRIDE;
+
+  //! Read the header information.
+  int RequestInformation(
+    vtkInformation* request, vtkInformationVector** inputVector,
+    vtkInformationVector* outputVector) VTK_OVERRIDE;
+
+  //! Read the voxel data.
+  int RequestData(
+    vtkInformation* request, vtkInformationVector** inputVector,
+    vtkInformationVector* outputVector) VTK_OVERRIDE;
+  //@}
+#else
+  //@{
+  //! Entry point for all pipeline requests.
+  virtual int ProcessRequest(
+    vtkInformation* request, vtkInformationVector** inputVector,
+    vtkInformationVector* outputVector);
+
   //! Read the header information.
   virtual int RequestInformation(
     vtkInformation* request, vtkInformationVector** inputVector,
@@ -244,6 +367,17 @@ protected:
     vtkInformation* request, vtkInformationVector** inputVector,
     vtkInformationVector* outputVector);
   //@}
+#endif
+
+  //@{
+  //! Read the overlays into an allocated vtkImageData object.
+  virtual bool ReadOverlays(vtkImageData *data);
+
+  //! Unpack overlay bits to build the overlay image.
+  void UnpackOverlay(
+    const void *filePtr, vtkIdType bitskip, vtkIdType count,
+    void *buffer, vtkIdType incr, int bit);
+  //@}
 
   //@{
   //! Read one file.  Specify the offset to the PixelData.
@@ -251,10 +385,18 @@ protected:
     const char *filename, int idx,
     unsigned char *buffer, vtkIdType bufferSize);
 
+  //! Clear or sign-extend any bits beyond BitsStored.
+  void MaskBits(void *buffer, vtkIdType bufferSize, int scalarSize,
+                int bitsStored, int pixelRepresentation);
+
   //! Unpack 1 bit to 8 bits or 12 bits to 16 bits.
   void UnpackBits(
     const void *source, void *buffer, vtkIdType bufferSize, int bits);
 
+  //! Unpack 4:2:2 color data.
+  void UnpackYBR422(
+    const void *source, void *buffer, vtkIdType bufferSize, vtkIdType rowlen);
+
   //! Read an DICOM file directly.
   virtual bool ReadFileNative(
     const char *filename, int idx,
@@ -267,9 +409,15 @@ protected:
   //@}
 
   //@{
+  //! Check if rescaling will change scalar type.
+  virtual int ComputeRescaledScalarType(
+    int scalarType, int bitsStored, int pixelRepresentation);
+
   //! Rescale the data in the buffer.
   virtual void RescaleBuffer(
-    int fileIdx, int frameIdx, void *buffer, vtkIdType bufferSize);
+    int fileIdx, int frameIdx, int fileType, int outputType,
+    int numFileComponents, int numComponents,
+    void *fileBuffer, void *outputBuffer, vtkIdType bufferSize);
 
   //! Convert buffer from YUV to RGB.
   virtual void YBRToRGB(
@@ -318,6 +466,12 @@ protected:
   //! The MedicalImageProperties, for compatibility with other readers.
   vtkMedicalImageProperties *MedicalImageProperties;
 
+  //! The default character set to use while parsing the file.
+  vtkDICOMCharacterSet DefaultCharacterSet;
+
+  //! Whether the default should override SpecificCharacterSet.
+  bool OverrideCharacterSet;
+
   //! The parser that is used to read the file.
   vtkDICOMParser *Parser;
 
@@ -342,6 +496,8 @@ protected:
   //! This indicates that the data must be rescaled.
   int NeedsRescale;
   int AutoRescale;
+  int FileScalarType;
+  int OutputScalarType;
 
   //! This indicates that the data must be converted to RGB.
   int NeedsYBRToRGB;
@@ -365,9 +521,21 @@ protected:
   //! The stack to load.
   char DesiredStackID[20];
 
+  //! Bitfield that says what overlays are present.
+  unsigned short OverlayBitfield;
+  bool UpdateOverlayFlag;
+
 private:
+#ifdef VTK_DELETE_FUNCTION
   vtkDICOMReader(const vtkDICOMReader&) VTK_DELETE_FUNCTION;
   void operator=(const vtkDICOMReader&) VTK_DELETE_FUNCTION;
+#elif __cplusplus >= 201103L
+  vtkDICOMReader(const vtkDICOMReader&) = delete;
+  void operator=(const vtkDICOMReader&) = delete;
+#else
+  vtkDICOMReader(const vtkDICOMReader&);
+  void operator=(const vtkDICOMReader&);
+#endif
 };
 
 #endif // vtkDICOMReader_h
