@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2015 David Gobbi
+  Copyright (c) 2012-2019 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -14,9 +14,11 @@
 #ifndef vtkDICOMSliceSorter_h
 #define vtkDICOMSliceSorter_h
 
-#include <vtkObject.h>
+#include "vtkObject.h"
 #include "vtkDICOMModule.h" // For export macro
 
+#include "vtkDICOMTag.h" // For vtkDICOMTag
+
 class vtkIntArray;
 class vtkStringArray;
 class vtkDICOMMetaData;
@@ -31,7 +33,11 @@ class VTKDICOM_EXPORT vtkDICOMSliceSorter : public vtkObject
 {
 public:
   vtkTypeMacro(vtkDICOMSliceSorter,vtkObject);
+#ifdef VTK_OVERRIDE
+  void PrintSelf(ostream& os, vtkIndent indent) VTK_OVERRIDE;
+#else
   void PrintSelf(ostream& os, vtkIndent indent);
+#endif
   static vtkDICOMSliceSorter *New();
 
   //@{
@@ -77,16 +83,30 @@ public:
   //@{
   //! Set the meta data for the DICOM files.
   /*!
-   *  The GetAttributeValue() method of vtkDICOMMetaData takes optional
-   *  file and frame indices, which specify the file and the frame within
-   *  that file to get the attribute from.  If you have a slice index rather
-   *  than a file index and frame index, then use the FileIndexArray and
+   *  The Get() method of vtkDICOMMetaData takes optional file and frame
+   *  indices, which specify the file and the frame within that file to
+   *  get the attribute from.  If you have a slice index rather than a file
+   *  file index and frame index, then use the FileIndexArray and
    *  FrameIndexArray to convert the slice index into file and frame indices.
    */
   void SetMetaData(vtkDICOMMetaData *meta);
   vtkDICOMMetaData *GetMetaData() { return this->MetaData; }
   //@}
 
+  //@{
+  //! Force repeated slices to be at different times (default: Off).
+  /*!
+   *  If this is on, then repeated slices at the same spatial position
+   *  will always be considered to be at different time points.  If this
+   *  is off, then the repeated slices will either become the temporal
+   *  dimension or the vector dimension, depending on the presence of
+   *  temporal attributes in the meta data.
+   */
+  vtkGetMacro(RepeatsAsTime, int);
+  vtkSetMacro(RepeatsAsTime, int);
+  vtkBooleanMacro(RepeatsAsTime, int);
+  //@}
+
   //@{
   //! Read the time dimension as scalar components (default: Off).
   /*!
@@ -113,6 +133,36 @@ public:
   vtkGetMacro(DesiredTimeIndex, int);
   //@}
 
+  //@{
+  //! Set the DICOM tag to use for time measurement.
+  /*!
+   *  This method can be used to explicitly set the tag to use for temporal
+   *  sorting.  By default (i.e. if this method is not used), the sorter
+   *  will search the meta-data for the following temporal tags and will
+   *  automatically apply them if present:
+   *  - TriggerTime (for cardiac images)
+   *  - EchoTime (for relaxometry)
+   *  - TemporalPositionIdentifier (fMRI)
+   */
+  void SetTimeTag(vtkDICOMTag tag);
+  vtkDICOMTag GetTimeTag() { return this->TimeTag; }
+  //@}
+
+  //@{
+  //! Set the DICOM sequence to use for timing information.
+  /*!
+   *  This is only used for enhanced multi-frame images.  If used, then
+   *  SetTimeTag() must also be used to specify which tag in the sequence
+   *  to use for temporal sorting.  By default, the following sequence/tag
+   *  conbinations are automatically detected and applied:
+   *  - CardiacSynchronizationSequence/NominalCardiacTriggerDelayTime
+   *  - TemporalPositionSequence/TemporalPositionTimeOffset
+   *  - FrameContentSequence/TemporalPositionIndex
+   *  - MREchoSequence/EffectiveEchoTime
+   */
+  void SetTimeSequence(vtkDICOMTag tag);
+  vtkDICOMTag GetTimeSequence() { return this->TimeSequence; }
+
   //@{
   //! Set whether to reverse the slice order.
   /*!
@@ -137,45 +187,50 @@ protected:
   vtkDICOMSliceSorter();
   ~vtkDICOMSliceSorter();
 
-  // Description:
   // Sort the input files, put the sort in the supplied arrays.
   virtual void SortFiles(vtkIntArray *fileArray, vtkIntArray *frameArray);
 
-  // Description:
   // The meta data for the image.
   vtkDICOMMetaData *MetaData;
 
-  // Description:
   // An array to convert slice indices to input files
   vtkIntArray *FileIndexArray;
 
-  // Description:
   // An array to convert slice indices to input frames
   vtkIntArray *FrameIndexArray;
 
-  // Description:
   // An array that holds the stack IDs.
   vtkStringArray *StackIDs;
 
-  // Description:
   // Time dimension variables.
+  int RepeatsAsTime;
   int TimeAsVector;
   int TimeDimension;
   int DesiredTimeIndex;
   double TimeSpacing;
 
-  // Description:
   // The stack to load.
   char DesiredStackID[20];
 
-  // Description:
   // Whether to reverse the slice order.
   int ReverseSlices;
   double SliceSpacing;
 
+  // The tags to use for time information.
+  vtkDICOMTag TimeTag;
+  vtkDICOMTag TimeSequence;
+
 private:
+#ifdef VTK_DELETE_FUNCTION
   vtkDICOMSliceSorter(const vtkDICOMSliceSorter&) VTK_DELETE_FUNCTION;
   void operator=(const vtkDICOMSliceSorter&) VTK_DELETE_FUNCTION;
+#elif __cplusplus >= 201103L
+  vtkDICOMSliceSorter(const vtkDICOMSliceSorter&) = delete;
+  void operator=(const vtkDICOMSliceSorter&) = delete;
+#else
+  vtkDICOMSliceSorter(const vtkDICOMSliceSorter&);
+  void operator=(const vtkDICOMSliceSorter&);
+#endif
 };
 
 #endif
