@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2015 David Gobbi
+  Copyright (c) 2012-2019 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -22,7 +22,11 @@ class VTKDICOM_EXPORT vtkDICOMSorter : public vtkDICOMFileSorter
 public:
   //@{
   vtkTypeMacro(vtkDICOMSorter,vtkDICOMFileSorter);
+#ifdef VTK_OVERRIDE
+  void PrintSelf(ostream& os, vtkIndent indent) VTK_OVERRIDE;
+#else
   void PrintSelf(ostream& os, vtkIndent indent);
+#endif
   static vtkDICOMSorter *New();
   //@}
 
@@ -39,8 +43,16 @@ protected:
   ~vtkDICOMSorter();
 
 private:
+#ifdef VTK_DELETE_FUNCTION
   vtkDICOMSorter(const vtkDICOMSorter&) VTK_DELETE_FUNCTION;
   void operator=(const vtkDICOMSorter&) VTK_DELETE_FUNCTION;
+#elif __cplusplus >= 201103L
+  vtkDICOMSorter(const vtkDICOMSorter&) = delete;
+  void operator=(const vtkDICOMSorter&) = delete;
+#else
+  vtkDICOMSorter(const vtkDICOMSorter&);
+  void operator=(const vtkDICOMSorter&);
+#endif
 };
 
 #endif
