@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2016 David Gobbi
+  Copyright (c) 2012-2019 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -11,22 +11,23 @@
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
-/*! \class vtkDICOMToRAS
- *  \brief Convert DICOM orientation to RAS orientation
+/**
+ * \class vtkDICOMToRAS
+ * \brief Convert DICOM orientation to RAS orientation
  *
- *  This class will modify an image and its position-and-orientation matrix
- *  so that the patient coordinate system follows the conventions of NIFTI
- *  and MINC, i.e. the x, y, and z axes will point in the right, anterior,
- *  and superior directions respectively.  Optionally, the row and column
- *  ordering of the image will also be modified so that the row direction
- *  preferentially points right or anterior, and the column direction
- *  preferentially points superior or anterior.
+ * This class will modify an image and its position-and-orientation matrix
+ * so that the patient coordinate system follows the conventions of NIFTI
+ * and MINC, i.e. the x, y, and z axes will point in the right, anterior,
+ * and superior directions respectively.  Optionally, the row and column
+ * ordering of the image will also be modified so that the row direction
+ * preferentially points right or anterior, and the column direction
+ * preferentially points superior or anterior.
  */
 
 #ifndef vtkDICOMToRAS_h
 #define vtkDICOMToRAS_h
 
-#include <vtkThreadedImageAlgorithm.h>
+#include "vtkThreadedImageAlgorithm.h"
 #include "vtkDICOMModule.h" // For export macro
 
 class vtkMatrix4x4;
@@ -40,10 +41,14 @@ public:
   vtkTypeMacro(vtkDICOMToRAS, vtkThreadedImageAlgorithm);
 
   //! Print information about this object.
-  virtual void PrintSelf(ostream& os, vtkIndent indent);
+#ifdef VTK_OVERRIDE
+  void PrintSelf(ostream& os, vtkIndent indent) VTK_OVERRIDE;
+#else
+  void PrintSelf(ostream& os, vtkIndent indent);
+#endif
 
   //@{
-  //! Perform RAS to DICOM instead of DICOM to RAS.
+  //! Reverse the filter: do RAS to DICOM instead of DICOM to RAS.
   void SetRASToDICOM(int v);
   void RASToDICOMOn() { this->SetRASToDICOM(1); }
   void RASToDICOMOff() { this->SetRASToDICOM(0); }
@@ -51,7 +56,7 @@ public:
   //@}
 
   //@{
-  //! Get a matrix to place the image within DICOM patient coords.
+  //! Set a matrix that places the image in DICOM patient coords.
   /*!
    *  This matrix is constructed from the ImageOrientationPatient
    *  and ImagePositionPatient meta data attributes.  See the
@@ -62,17 +67,18 @@ public:
   //@}
 
   //@{
-  //! Get a matrix that follows the RAS convention that can be used
+  //! Get a matrix that places the image in RAS coordinates.
   /*!
-   *  with the output image.  This matrix is only valid for the output
-   *  image, it is not valid for the input image.
+   *  This matrix is meant to be used with the output image (unless
+   *  the method RASToDICOMOn() has been called to reverse the operation
+   *  of the filter).
    */
-  void SetRASMatrix(vtkMatrix4x4 *matrix);
   vtkMatrix4x4 *GetRASMatrix() { return this->RASMatrix; }
+  void SetRASMatrix(vtkMatrix4x4 *matrix);
   //@}
 
   //@{
-  //! Update the RAS matrix without updating the output data.
+  //! Update the matrix without updating the output data.
   /*!
    *  This requires that an input has been set, because the origin,
    *  spacing, and extent of the input data must be known in order
@@ -82,13 +88,15 @@ public:
   //@}
 
   //@{
-  //! Specify whether the RAS matrix should include the position in its
+  //! Set whether the RAS matrix should hava a non-zero final column.
   /*!
-   *  final column (NIFTI style), or whether the final column should be
-   *  zero and the position instead used to compute a new Origin for the
-   *  output image.  If this option is On, then the Origin of the output
-   *  image will be (0,0,0), and if it is Off, then the position stored
-   *  the matrix will be (0,0,0).
+   *  By default, the RAS matrix produced by this filter will have a
+   *  position coordinate in its final column (NIfTI style).
+   *  Alternatively, this filter can set the final column to zero and
+   *  use the position to compute a new Origin for the output image.
+   *  If this option is On, then the Origin of the output image will
+   *  be (0,0,0), and if it is Off, then the final column of the matrix
+   *  will be (0,0,0).
    */
   void SetRASMatrixHasPosition(int v);
   void RASMatrixHasPositionOn() { this->SetRASMatrixHasPosition(1); }
@@ -97,8 +105,9 @@ public:
   //@}
 
   //@{
-  //! Allow the columns to be reordered so that columns with higher indices
+  //! Allow the columns of the image to be reordered.
   /*!
+   *  The columns can be reordered so that columns with higher indices
    *  are further to the right or further anterior.  Note that if you turn
    *  this option on, then you are implicitly allowing slice reordering to
    *  occur as well.  If you turn both ColumnReordering and SliceReordering
@@ -111,8 +120,9 @@ public:
   //@}
 
   //@{
-  //! Allow the rows to be reordered so that rows with higher indices
+  //! Allow the rows of the image to be reordered.
   /*!
+   *  The rows can be reordered so that rows with higher indices
    *  are further superior or further anterior.  Note that if you turn
    *  this option on, then you are implicitly allowing slice reordering to
    *  occur as well.  If you turn both ColumnReordering and SliceReordering
@@ -138,22 +148,41 @@ protected:
    */
   void ComputeMatrix(int extent[6], double spacing[3], double origin[3]);
 
-  virtual int RequestInformation(
+#ifdef VTK_OVERRIDE
+  int RequestInformation(
+    vtkInformation* request, vtkInformationVector** inputVector,
+    vtkInformationVector* outputVector) VTK_OVERRIDE;
+
+  int RequestUpdateExtent(
+    vtkInformation* request, vtkInformationVector** inputVector,
+    vtkInformationVector* outputVector) VTK_OVERRIDE;
+
+  int RequestData(
+    vtkInformation* request, vtkInformationVector** inputVector,
+    vtkInformationVector* outputVector) VTK_OVERRIDE;
+
+  void ThreadedRequestData(
+    vtkInformation *request, vtkInformationVector **inputVector,
+    vtkInformationVector *outputVector, vtkImageData ***inData,
+    vtkImageData **outData, int ext[6], int id) VTK_OVERRIDE;
+#else
+  int RequestInformation(
     vtkInformation* request, vtkInformationVector** inputVector,
     vtkInformationVector* outputVector);
 
-  virtual int RequestUpdateExtent(
+  int RequestUpdateExtent(
     vtkInformation* request, vtkInformationVector** inputVector,
     vtkInformationVector* outputVector);
 
-  virtual int RequestData(
+  int RequestData(
     vtkInformation* request, vtkInformationVector** inputVector,
     vtkInformationVector* outputVector);
 
-  virtual void ThreadedRequestData(
+  void ThreadedRequestData(
     vtkInformation *request, vtkInformationVector **inputVector,
     vtkInformationVector *outputVector, vtkImageData ***inData,
     vtkImageData **outData, int ext[6], int id);
+#endif
 
   vtkMatrix4x4 *PatientMatrix;
   vtkMatrix4x4 *RASMatrix;
@@ -167,8 +196,16 @@ protected:
   double Matrix[16];
 
 private:
+#ifdef VTK_DELETE_FUNCTION
   vtkDICOMToRAS(const vtkDICOMToRAS&) VTK_DELETE_FUNCTION;
   void operator=(const vtkDICOMToRAS&) VTK_DELETE_FUNCTION;
+#elif __cplusplus >= 201103L
+  vtkDICOMToRAS(const vtkDICOMToRAS&) = delete;
+  void operator=(const vtkDICOMToRAS&) = delete;
+#else
+  vtkDICOMToRAS(const vtkDICOMToRAS&);
+  void operator=(const vtkDICOMToRAS&);
+#endif
 };
 
 #endif // vtkDICOMToRAS_h
