@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2015 David Gobbi
+  Copyright (c) 2012-2019 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -14,8 +14,8 @@
 #ifndef vtkDICOMUtilities_h
 #define vtkDICOMUtilities_h
 
-#include <vtkObject.h>
-#include <vtkStdString.h> // For std::string
+#include "vtkObject.h"
+#include "vtkStdString.h" // For std::string
 #include "vtkDICOMModule.h" // For export macro
 #include "vtkDICOMTag.h" // For method parameter
 
@@ -33,7 +33,11 @@ public:
   vtkTypeMacro(vtkDICOMUtilities, vtkObject);
 
   //! Print a summary of the contents of this object.
+#ifdef VTK_OVERRIDE
+  void PrintSelf(ostream& os, vtkIndent indent) VTK_OVERRIDE;
+#else
   void PrintSelf(ostream& os, vtkIndent indent);
+#endif
   //@}
 
   //@{
@@ -96,16 +100,17 @@ public:
    */
   static long long ConvertDateTime(const char *datetime);
 
-  //! Get the current UTC time in microseconds and an offset to localtime.
+  //! Get the current time in microseconds.
   /*!
    *  The time is in microseconds since the UNIX epoch (00:00:00 UTC on
-   *  1 Jan 1970).  The offset can be added to the returned UTC time in
-   *  order to get the local time.  If you do not need the offset, you
-   *  can pass a null pointer.
+   *  1 Jan 1970).
    */
-  static long long GetUTC(long long *offset);
+  static long long GetUniversalTime();
   //@}
 
+  //! Deprecated method, do not use.
+  static long long GetUTC(long long *offset);
+
   //@{
   //! Check if the specified file is a DICOM file.
   /*!
@@ -175,6 +180,17 @@ public:
     const char *pattern, size_t pl, const char *val, size_t vl);
   //@}
 
+  //@{
+  //! Check that a string is valid utf-8.
+  /*!
+   *  The string is valid if it contains no NULL bytes and if it contains
+   *  only valid 1-byte, 2-byte, 3-byte, or 4-byte utf-8 sequences.  Any
+   *  overlength sequences or sequences that encode values above U+10FFFF
+   *  or between U+D800 and U+DFFF are considered invalid.
+   */
+  static bool IsValidUTF8(const char *text, size_t l);
+  //@}
+
   //@{
   //! Get the name associated with the given UID.
   static const char *GetUIDName(const char *uid);
@@ -187,13 +203,23 @@ protected:
   vtkDICOMUtilities() {}
   ~vtkDICOMUtilities() {}
 
+  static long long GetLocalOffset(long long t);
+
   static char UIDPrefix[64];
   static char ImplementationClassUID[65];
   static char ImplementationVersionName[17];
 
 private:
+#ifdef VTK_DELETE_FUNCTION
   vtkDICOMUtilities(const vtkDICOMUtilities&) VTK_DELETE_FUNCTION;
   void operator=(const vtkDICOMUtilities&) VTK_DELETE_FUNCTION;
+#elif __cplusplus >= 201103L
+  vtkDICOMUtilities(const vtkDICOMUtilities&) = delete;
+  void operator=(const vtkDICOMUtilities&) = delete;
+#else
+  vtkDICOMUtilities(const vtkDICOMUtilities&);
+  void operator=(const vtkDICOMUtilities&);
+#endif
 };
 
 #endif /* vtkDICOMUtilities_h */
