@@ -12,30 +12,31 @@
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
-/*! \class vtkNIFTIHeader
- *  \brief Store NIfTI header information.
+/**
+ * \class vtkNIFTIHeader
+ * \brief Store NIfTI header information.
  *
- *  This class stores the header of a NIfTI file in a VTK-friendly format.
- *  By using this class, it is possible to specify the header information
- *  that will be stored in a file written by the vtkNIFTIWriter.  Note
- *  that the SForm and QForm orientation information in this class will be
- *  ignored by the writer if an SForm and QForm have been explicitly set
- *  via the writer's SetSForm and SetQForm methods.  Also note that all
- *  info like Dim, PixDim, DataType, etc. will be ignored by the writer
- *  because this information must instead be taken from the vtkImageData
- *  information.  Finally, note that the vtkNIFTIWriter will ignore the
- *  Descrip field, since it has its own SetDescription method.
+ * This class stores the header of a NIfTI file in a VTK-friendly format.
+ * By using this class, it is possible to specify the header information
+ * that will be stored in a file written by the vtkNIFTIWriter.  Note
+ * that the SForm and QForm orientation information in this class will be
+ * ignored by the writer if an SForm and QForm have been explicitly set
+ * via the writer's SetSForm and SetQForm methods.  Also note that all
+ * info like Dim, PixDim, DataType, etc. will be ignored by the writer
+ * because this information must instead be taken from the vtkImageData
+ * information.  Finally, note that the vtkNIFTIWriter will ignore the
+ * Descrip field, since it has its own SetDescription method.
  *
- *  This class was contributed to VTK by the Calgary Image Processing and
- *  Analysis Centre (CIPAC).
+ * This class was contributed to VTK by the Calgary Image Processing and
+ * Analysis Centre (CIPAC).
  *
- *  \sa vtkNIFTIReader, vtkNIFTIWriter
+ * \sa vtkNIFTIReader, vtkNIFTIWriter
  */
 
 #ifndef vtkNIFTIHeader_h
 #define vtkNIFTIHeader_h
 
-#include <vtkObject.h>
+#include "vtkObject.h"
 #include "vtkDICOMModule.h" // For export macro
 
 struct nifti_1_header;
@@ -162,7 +163,11 @@ public:
   vtkTypeMacro(vtkNIFTIHeader, vtkObject);
 
   //! Print information about this object.
-  virtual void PrintSelf(ostream& os, vtkIndent indent);
+#ifdef VTK_OVERRIDE
+  void PrintSelf(ostream& os, vtkIndent indent) VTK_OVERRIDE;
+#else
+  void PrintSelf(ostream& os, vtkIndent indent);
+#endif
 
   //@{
   //! Get the magic number for the NIFTI file as a null-terminated string.
@@ -359,6 +364,12 @@ public:
   void GetHeader(nifti_2_header *hdr);
   //@}
 
+  //@{
+  //! Byte-swap the nifti header struct.
+  static void ByteSwapHeader(nifti_1_header *hdr);
+  static void ByteSwapHeader(nifti_2_header *hdr);
+  //@}
+
 protected:
   vtkNIFTIHeader();
   ~vtkNIFTIHeader();
@@ -402,8 +413,16 @@ protected:
   void SetStringValue(char *x, const char *y, size_t n);
 
 private:
+#ifdef VTK_DELETE_FUNCTION
   vtkNIFTIHeader(const vtkNIFTIHeader&) VTK_DELETE_FUNCTION;
   void operator=(const vtkNIFTIHeader&) VTK_DELETE_FUNCTION;
+#elif __cplusplus >= 201103L
+  vtkNIFTIHeader(const vtkNIFTIHeader&) = delete;
+  void operator=(const vtkNIFTIHeader&) = delete;
+#else
+  vtkNIFTIHeader(const vtkNIFTIHeader&);
+  void operator=(const vtkNIFTIHeader&);
+#endif
 };
 
 #endif // vtkNIFTIHeader_h
