@@ -12,27 +12,28 @@
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
-/*! \class vtkNIFTIWriter
- *  \brief Write NIfTI-1 and NIfTI-2 medical image files
+/**
+ * \class vtkNIFTIWriter
+ * \brief Write NIfTI-1 and NIfTI-2 medical image files
  *
- *  This class writes NIFTI files, either in .nii format or as separate
- *  .img and .hdr files.  If told to write a file that ends in ".gz",
- *  then the writer will automatically compress the file with zlib.
- *  Images of type unsigned char that have 3 or 4 scalar components
- *  will automatically be written as RGB or RGBA respectively.  Images
- *  of type float or double that have 2 components will automatically be
- *  written as complex values.
+ * This class writes NIFTI files, either in .nii format or as separate
+ * .img and .hdr files.  If told to write a file that ends in ".gz",
+ * then the writer will automatically compress the file with zlib.
+ * Images of type unsigned char that have 3 or 4 scalar components
+ * will automatically be written as RGB or RGBA respectively.  Images
+ * of type float or double that have 2 components will automatically be
+ * written as complex values.
  *
- *  This class was contributed to VTK by the Calgary Image Processing and
- *  Analysis Centre (CIPAC).
+ * This class was contributed to VTK by the Calgary Image Processing and
+ * Analysis Centre (CIPAC).
  *
- *  \sa vtkNIFTIReader
+ * \sa vtkNIFTIReader
  */
 
 #ifndef vtkNIFTIWriter_h
 #define vtkNIFTIWriter_h
 
-#include <vtkImageWriter.h>
+#include "vtkImageWriter.h"
 #include "vtkDICOMModule.h" // For export macro
 
 class vtkMatrix4x4;
@@ -41,12 +42,23 @@ class vtkNIFTIHeader;
 class VTKDICOM_EXPORT vtkNIFTIWriter : public vtkImageWriter
 {
 public:
+
+  //! Endianness of output file.
+  enum EndianEnum {
+    BigEndian = 0,
+    LittleEndian = 1
+  };
+
   //! Static method for construction.
   static vtkNIFTIWriter *New();
   vtkTypeMacro(vtkNIFTIWriter, vtkImageWriter);
 
   //! Print information about this object.
-  virtual void PrintSelf(ostream& os, vtkIndent indent);
+#ifdef VTK_OVERRIDE
+  void PrintSelf(ostream& os, vtkIndent indent) VTK_OVERRIDE;
+#else
+  void PrintSelf(ostream& os, vtkIndent indent);
+#endif
 
   //@{
   //! Set the version number for the NIfTI file format to use.
@@ -133,7 +145,7 @@ public:
   /*!
    *  Unlike the qform matrix, the sform matrix can contain scaling
    *  information.  Before being stored in the NIFTI header, the
-   *  first three columns of the matrix will be multipled by the voxel
+   *  first three columns of the matrix will be multiplied by the voxel
    *  spacing. In the NIFTI header, the sform_code will be set to 2.
    */
   void SetSFormMatrix(vtkMatrix4x4 *);
@@ -152,6 +164,24 @@ public:
   vtkNIFTIHeader *GetNIFTIHeader();
   //@}
 
+  //@{
+  //! Set the byte order for the files (default: little endian).
+  /*!
+   *  These methods allow the byte order of the NIFTI file to be set.
+   *  Originally, Analyze files (and NIFTI, by extension) were meant to
+   *  be written using the native endianness of the system.  However,
+   *  with the overwhelming preference for little-endian architectures
+   *  on image processing workstations, defaulting to little endian
+   *  files (even on big-endian systems) provides better compatibility.
+   */
+  vtkSetMacro(DataByteOrder, EndianEnum);
+  void SetDataByteOrderToBigEndian() {
+    this->SetDataByteOrder(BigEndian); }
+  void SetDataByteOrderToLittleEndian() {
+    this->SetDataByteOrder(LittleEndian); }
+  vtkGetMacro(DataByteOrder, EndianEnum);
+  //@}
+
 protected:
   vtkNIFTIWriter();
   ~vtkNIFTIWriter();
@@ -160,9 +190,15 @@ protected:
   int GenerateHeader(vtkInformation *info, bool singleFile);
 
   //! The main execution method, which writes the file.
-  virtual int RequestData(vtkInformation *request,
-                          vtkInformationVector** inputVector,
-                          vtkInformationVector* outputVector);
+#ifdef VTK_OVERRIDE
+  int RequestData(vtkInformation *request,
+                  vtkInformationVector** inputVector,
+                  vtkInformationVector* outputVector) VTK_OVERRIDE;
+#else
+  int RequestData(vtkInformation *request,
+                  vtkInformationVector** inputVector,
+                  vtkInformationVector* outputVector);
+#endif
 
   //! Make a new filename by replacing extension "ext1" with "ext2".
   /*!
@@ -199,9 +235,20 @@ protected:
   //! Use planar RGB instead of the default (packed).
   bool PlanarRGB;
 
+  //! Whether the file should be little endian.
+  EndianEnum DataByteOrder;
+
 private:
+#ifdef VTK_DELETE_FUNCTION
   vtkNIFTIWriter(const vtkNIFTIWriter&) VTK_DELETE_FUNCTION;
   void operator=(const vtkNIFTIWriter&) VTK_DELETE_FUNCTION;
+#elif __cplusplus >= 201103L
+  vtkNIFTIWriter(const vtkNIFTIWriter&) = delete;
+  void operator=(const vtkNIFTIWriter&) = delete;
+#else
+  vtkNIFTIWriter(const vtkNIFTIWriter&);
+  void operator=(const vtkNIFTIWriter&);
+#endif
 };
 
 #endif // vtkNIFTIWriter_h
